var _ = require('lodash');
var Po = require('pofile');
function referenceMatches(oldRef, newRef) {
    return _(oldRef).split(':').first() === _(newRef).split(':').first();
}
function mergeReferences(oldRefs, newRefs) {
    var _newRefs = _(newRefs);
    return _(oldRefs)
        .reject(function (oldRef) {
        return _newRefs.some(function (newRef) { return referenceMatches(oldRef, newRef); });
    })
        .concat(newRefs)
        .uniq()
        .sort()
        .value();
}
var Registry = (function () {
    function Registry() {
        this.stringsList = [];
    }
    Registry.prototype.addGetTextStrings = function (strings) {
        this.stringsList.push(strings);
    };
    // Merge two po strings, please note that this function will modify the target instance
    Registry.prototype.merge = function (target, from) {
        _.forIn(from, function (contextObject, msgid) {
            if (!target[msgid]) {
                target[msgid] = {};
            }
            _.forIn(contextObject, function (item, context) {
                var existing = target[msgid][context];
                if (!existing) {
                    target[msgid][context] = item;
                }
                else {
                    existing.comments = _.uniq(existing.comments.concat(item.comments)).sort();
                    existing.references = mergeReferences(item.references, existing.references);
                }
            });
        });
    };
    Registry.prototype.toString = function () {
        if (this.stringsList.length) {
            this.strings = this.stringsList[0];
            for (var i = 1; i < this.stringsList.length; i++) {
                this.merge(this.strings, this.stringsList[i]);
            }
            // Clear
            this.stringsList.length = 0;
        }
        var catalog = new Po();
        catalog.headers = {
            'Content-Type': 'text/plain; charset=UTF-8',
            'Content-Transfer-Encoding': '8bit',
            'Project-Id-Version': ''
        };
        if (this.strings) {
            _.forIn(this.strings, function (msg, msgid) {
                var contexts = _.keys(msg).sort();
                _.forEach(contexts, function (context) {
                    catalog.items.push(msg[context]);
                });
            });
        }
        catalog.items.sort(function (a, b) {
            return a.msgid.localeCompare(b.msgid);
        });
        return catalog.toString();
    };
    return Registry;
})();
exports.Registry = Registry;
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = Registry;
//# sourceMappingURL=registry.js.map